/********************************************************************/
/*	EASYFSEL.C														*/
/*																	*/
/*	Routine zum leichten und AES-Versions abhngigen Aufruf der		*/
/*	GEM-Fileselectorbox mit Auswertung des 'FSEL' - Cookies.		*/
/*	Auerdem sind einige Routinen zum Umgang mit Dateien enthalten,	*/
/*	sowie zum Suchen eines Cookies.									*/
/*																	*/
/*	Version : 1.27													*/
/*	Datum   : 15.06.1992											*/
/*	Autor		: Andreas Papula (falls nicht anders angegeben)		*/
/*																	*/
/*  Erweitert und gekrzt fr WIN_LNK von Dimitri Junker			*/
/*	Copyright 1992 by MAXON Computer GmbH							*/
/********************************************************************/

/*------------------------------------------------------------------*/
/*	Include-Files einbinden.										*/
/*------------------------------------------------------------------*/

#include <easyfsel.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

/* -------------------------------------------------------------------- */
/*       Makros. & Konstanten                                           */
/* -------------------------------------------------------------------- */

#define _sysbase	(0x4f2l)

/* -------------------------------------------------------------------- */
/* 		lokale Funktionsprototypen										*/
/* -------------------------------------------------------------------- */
long * get_cookie_ptr(long cookie_name);


/*------------------------------------------------------------------*/
/*	VOID build_filename(BYTE *dest, BYTE *pfad, BYTE *dateiname,BOOLEAN noFinPf)	*/
/*																	*/
/*	Bastelt Pfad- und Dateinamen zusammen.							*/
/*																	*/
/*	Parameter:	dest      = Zielstring.								*/
/*				pfad      = Pfadname.								*/
/*				dateiname = Dateiname.								*/
/*				noFinPf:  =no File in Pfadname						*/
/* bisher ging build_filename davon aus, da evtl. einen Pfadnamen	*/
/*	enthlt, der dann verworfen wird. Dies wird jetzt mit 			*/
/*	noFinPf=FALSE erreicht. Sonst wird immer an pfad angehngt,		*/
/*	ggf auch zuerst ein '\\'										*/
/*	Rckgabe :Das Ergebnis befindet sich in dest.					*/
/*------------------------------------------------------------------*/

VOID build_filename(BYTE *dest, BYTE *pfad, BYTE *dateiname,BOOLEAN noFinPf)
{
	BYTE *xyz;
	size_t len;

	strcpy(dest, pfad);
	if(noFinPf &&*dest)
	{
		len=strlen(dest);
		if(dest[len-1]!='\\')
			strcat(dest,"\\");
	}
	xyz = strrchr(dest, (int) '\\');
	strcpy(++xyz, dateiname);
}

/*------------------------------------------------------------------*/
/*	void trenn_pfad(char *pfad,char *name,char *file)				*/
/*																	*/
/*	Bastelt Pfad- und Dateinamen auseinander, 						*/
/*	Gegenteil von build_filename.									*/
/*	dabei endet der Pfad mit '\\'									*/
/*																	*/
/*	Parameter:			file	= File mit Pfad+Name				*/
/*						pfad    = Pfadname.							*/
/*						name 	= Dateiname.						*/
/*			file und pfad drfen gleich sein						*/
/*	war nicht Teil des Originals, also von DYJ						*/
/*------------------------------------------------------------------*/

void trenn_pfad(char *pfad,char *name,const char *file)
{ 	char *ptr;
	if(file!=pfad)
		strcpy(pfad,file);
	if((ptr=strrchr(pfad,'\\'))==NULL)
		ptr=pfad;					/* dann ist wohl alles nur Name	*/
	else
		ptr++;
	strcpy(name,ptr);
	*ptr='\0';
}
	

/*------------------------------------------------------------------*/
/*	BOOLEAN get_cookie(long cookie_name, LONG *cookie_value)		*/
/*																	*/
/*	Sucht nach einem Cookie 'cookie_name' und liefert dessen Wert	*/
/*	in 'cookie_value' zurck.										*/
/*																	*/
/*	Parameter:Name des Cookies und einen Zeiger auf den Wert des	*/
/*						Cookies.									*/
/*	Rckgabe :TRUE, falls der Cookie gefunden wurde, FALSE falls	*/
/*						nicht.										*/
/*					In 'cookie_value' steht der Wert des Cookies.	*/
/*					cookie_value kann NULL sein, dann wird nur die	*/
/*					Existenz des Cookies getestet					*/
/*------------------------------------------------------------------*/

BOOLEAN get_cookie(long cookie_name, LONG *cookie_value)
{
	LONG *cookie;
	int ret=1;
	
	/* Zuerst einen Zeiger auf den Cookie holen */

	cookie = get_cookie_ptr(cookie_name);

	/* Ist der Cookiejar vorhanden ? */
    
	if(!cookie)
		ret=0;
	else
	{	if(!cookie[0])
			ret=0;
		else
		{	if(cookie_value)
				*cookie_value=cookie[1];
		}
	}
	return(ret);
}
/*------------------------------------------------------------------*/
/*	long get_cookie_jar()											*/
/*																	*/
/*	Ermittelt die Adresse des Cookie Jar.							*/
/*																	*/
/*	Parameter:keine				 									*/
/*	Rckgabe :Adresse, falls der Cookie Jar gefunden wurde,			*/
/*			  NULL falls kein Cookie-Jar							*/
/*  Besonderheit: mu per Supexec aufgerufen werden					*/
/*	(c) by Dimitri Junker 1996										*/
/*------------------------------------------------------------------*/

long  get_cookie_jar()
{
	LONG *cookiejar;

	cookiejar = *((LONG **) 0x5a0L);
	return((long)cookiejar);
}
/*------------------------------------------------------------------*/
/*	long *get_cookie_ptr(BYTE *cookie_name)							*/
/*																	*/
/*	Sucht nach einem Cookie 'cookie_name' und liefert dessen 		*/
/*	Adresse zurck.													*/
/*																	*/
/*	Parameter:Name des Cookies 										*/
/*	Rckgabe :Adresse, falls der Cookie gefunden wurde,				*/
/*			  Adresse des NULL-cookie falls nicht vorhanden			*/
/*			  NULL falls kein Cookie-Jar							*/
/*	(c) by Dimitri Junker 1996										*/
/*------------------------------------------------------------------*/

long * get_cookie_ptr(long cookie_name)
{
	LONG *cookiejar;

	/* Zuerst einen Zeiger auf den Cookiejar holen */

	cookiejar= (long *)Supexec(get_cookie_jar);

	/* Ist der Cookiejar vorhanden ? */
    
	if(!cookiejar)
		return(NULL);
	else
	{
		do
		{
			/* Ist es unser Cookie ? */
			if(*cookiejar== cookie_name)
				break;
			else
			{
				/* nchster Cookie aus Jar */
				cookiejar = &(cookiejar[2]);
			}
		} 
		while(cookiejar[0]);  /* solange nicht NULL-Cookie */
    
		return(cookiejar);;
  }
}

/*------------------------------------------------------------------*/
/*	int set_cookie(long cookie_name, LONG cookie_value)		*/
/*																	*/
/*	Setzt einen Cookie 'cookie_name' auf den Wert cookie_value		*/
/*																	*/
/*	Parameter:Name des Cookies und den Wert des	Cookies.			*/
/*	Rckgabe :0, falls der Cookie gesetzt werden konnte, 			*/
/*			  ERR_FS_KJAR falls	kein Cookie Jar vorhanden 			*/
/*			  ERR_FS_COOK falls cookie_name bereits vorhanden		*/
/*			  ERR_FS_SPEI falls nicht genug Speicher fr greren Jar*/
/*	(c) by Dimitri Junker 1996										*/
/*------------------------------------------------------------------*/

int set_cookie(long cookie_name, LONG cookie_value)
{	int err=0;
	LONG *cookie,*old_jar,*new_jar,len;
	/* Prfen ob er bereits existiert:	*/
	if((cookie=get_cookie_ptr(cookie_name))==NULL)
		err=ERR_FS_KJAR;
	else
	{	if(*cookie)
			err=ERR_FS_COOK;
	}
	/* ggf greren Jar anlegen		*/
	if(!err)
	{	old_jar=(long *)Supexec(get_cookie_jar);
		len=(long)cookie-(long)old_jar;
		if(len<(cookie[1]-1)*8l)
		{	cookie[2]=cookie[0];
			cookie[3]=cookie[1];
		}
		else	/* vergrern	*/
		{	len=cookie[1]+20;
			if((new_jar=malloc(len*8l))==NULL)
				err=ERR_FS_SPEI;
			else
			{	memcpy(new_jar,old_jar,cookie[1]*8l);
				cookie=&new_jar[((int)cookie[1]-1)*2];
				cookie[2]=0l;
				cookie[3]=len;
			}
		}
	}
	if(!err)
	{	cookie[0]=cookie_name;
		cookie[1]=cookie_value;
	}
	return(err);
}
/*------------------------------------------------------------------*/
/*	void del_cookie(long cookie_name)								*/
/*																	*/
/*	Lscht einen Cookie 'cookie_name'								*/
/*																	*/
/*	Parameter:Name des Cookies										*/
/*	(c) by Dimitri Junker 1996										*/
/*------------------------------------------------------------------*/

void del_cookie(long cookie_name)
{	LONG *cookie;
	/* Prfen ob er bereits existiert:	*/
	if((cookie=get_cookie_ptr(cookie_name))!=NULL)
	{	while(cookie[0])
		{	cookie[0]=cookie[2];
			cookie[1]=cookie[3];
			cookie=&cookie[2];
		}
	}
}

/*------------------------------------------------------------------*/
/*	int cmp_maske(char *str,char *maske);							*/
/*																	*/
/*	Prft ob str mit maske vertrglich ist.							*/
/*																	*/
/*	Parameter:	str ein String										*/
/*				maske die maske, mit * und ?						*/
/*			ign_case: wenn <>0 wird Gro-/Kleinschreibung ignoriert */
/*  Achtung *? ist nicht =* sondern irgendwas gefolgt von  '?'		*/
/*	Rckgabe :Die Funktion liefert als Ergebnis 1, wenn 			*/
/*             beide vertrglich sind, sonst 0						*/
/*	(c) by Dimitri Junker 1996										*/
/*------------------------------------------------------------------*/
int cmp_maske(char *str,char *maske,int ign_case)
{	char *s_ptr;
	int gleich=0;
	
/*DYJ 0.22e Start*/
	if(!strcmp(maske,"*.*"))
		return 1;
/*DYJ  Ende; */
	switch((int)*maske)
	{	case '*':
			s_ptr=str;
			while((s_ptr=strchr(s_ptr,(int)maske[1]))!=NULL && !gleich)
				gleich=cmp_maske(s_ptr++,&maske[1],ign_case);
			break;
		case '?':
			if(*str)
				gleich=cmp_maske(&str[1],&maske[1],ign_case);
			break;
		case '\0':
			gleich=(*str=='\0');
			break;
		default:
			if(*str==*maske || (ign_case&& (toupper(*str)==toupper(*maske))))
				gleich=cmp_maske(&str[1],&maske[1],ign_case);
	}
	return(gleich);
}

/*----------------------- Ende der Datei. --------------------------*/
	